/*-------------------------------------------------------------------------------------
// Copyright 2006, Texas Instruments Incorporated
//
// This program has been modified from its original operation by Texas Instruments
// to do the following:
//
// 1. Fixed caching of junk values for hostnames
//
// THIS MODIFIED SOFTWARE AND DOCUMENTATION ARE PROVIDED
// "AS IS," AND TEXAS INSTRUMENTS MAKES NO REPRESENTATIONS
// OR WARRENTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
// TO, WARRANTIES OF MERCHANTABILITY OR FITNESS FOR ANY
// PARTICULAR PURPOSE OR THAT THE USE OF THE SOFTWARE OR
// DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY PATENTS,
// COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.
// See The GNU General Public License for more details.
//
// These changes are covered under version 2 of the GNU General Public License,
// dated June 1991.
//-------------------------------------------------------------------------------------*/

#define NUM_RRS 5
/*****************************************************************************/
struct dns_rr{//
  char name[NAME_SIZE] __attribute__ ((aligned));
  uint16 type;
  uint16 class;
  uint32 ttl;
  uint16 rdatalen;
  char data[NAME_SIZE] __attribute__ ((aligned));
};
/*****************************************************************************/

#define GET_QUESTION(x) ((x & 0x8000) >> 15)
#define GET_OPCODE(x) ((x & 0x7800) >> 11)
#define GET_AUTHORATIVE(x) ((x & 0x400) >> 10)
#define GET_TRUNCATED(x) ((x & 0x200) >> 9)
#define GET_WANT_RECURSION(x) ((x & 0x100) >> 8)
#define GET_RECURSION_AVAIL(x) ((x & 0x80) >> 7)
#define GET_UNUSED(x) ((x & 0x70) >> 4)
#define GET_RCODE(x) (x & 0xf)

#define SET_QUESTION(x) (x|=0x8000)
#define SET_RCODE(x,y) (x|=(y & 0xf))

/*****************************************************************************/
struct dns_header_s{
  uint16 id;
  uint16 flags;
  uint16 qdcount;
  uint16 ancount;
  uint16 nscount;
  uint16 arcount;
};
/*****************************************************************************/
struct dns_message{
  struct dns_header_s header;
  struct dns_rr question[NUM_RRS];
  struct dns_rr answer[NUM_RRS];
};
/*****************************************************************************/
typedef struct dns_request_s{
  char cname[NAME_SIZE] __attribute__ ((aligned));
  char ip[20] __attribute__ ((aligned));
  int cache; 
  int ttl;
  int time_pending; /* request age in seconds */

  /* the actual dns request that was recieved */
  struct dns_message message;

  /* where the request came from */
  struct in_addr src_addr __attribute__ ((aligned));
  int src_port;

  /* the orginal packet */
  char original_buf[MAX_PACKET_SIZE];
  int numread;
  char *here;

  /* next node in list */
  struct dns_request_s *next;
}dns_request_t;
/*****************************************************************************/
/* TYPE values */
enum{ A = 1,      /* a host address */
	NS,       /* an authoritative name server */
	MD,       /* a mail destination (Obsolete - use MX) */
	MF,       /* */
	CNAME,    /* the canonical name for an alias */
	SOA,      /* marks the start of a zone of authority  */
	MB,       /* a mailbox domain name (EXPERIMENTAL) */
	MG,       /* */
	MR,       /* */
	NUL,      /* */
	WKS,      /* a well known service description */
	PTR,      /* a domain name pointer */
	HINFO,    /* host information */
	MINFO,    /* mailbox or mail list information */
	MX,       /* mail exchange */
	TXT,      /* text strings */

	AAA = 0x1c /* IPv6 A */
	};

/* CLASS values */
enum{
  IN = 1,         /* the Internet */
    CS,
    CH,
    HS
};

/* OPCODE values */
enum{
  QUERY,
    IQUERY,
    STATUS
};

/* Response codes */
enum {
	DNS_NO_ERROR,
	DNS_FMT_ERROR,
	DNS_SRVR_FAIL,
	DNS_NAME_ERR,
	DNS_NOT_IMPLEMENTED,
	DNS_REFUSED
};
	
