/*
 *
 * docint.h
 * Description:
 * declaration of types used in the interrupt cause-detection module
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef __DOCINT_H_
#define __DOCINT_H_

#ifndef DI_NAME
typedef enum 
{
    DI_MINOR_START,                 /* First!!! */
    DI_MAC_ERR = DI_MINOR_START,    /* 0 */
    DI_MAC_UCD,                     /* 1 */
    DI_MAC_US_FW,                   /* 2 */
    DI_MAC_DS_FW,                   /* 3 */
    DI_MAC_PHY,                     /* 4 */
    DI_PHY_PGA_GRT,                 /* 5 */
    DI_REMOTE_PHY,                  /* 6 */
    DI_PHY_MPT,                     /* 7 */
    DI_COUNT                        /* Last!!! */
} DiDevices_e;
#define	DI_NAME		"DOCINT"
#endif

typedef struct intinfo 
{
	short	ii_inum;
	char	ii_masked;	
	char	ii_clear;
} ii_t;

#ifdef __KERNEL__
/* Device structure */
typedef struct docint {
	ii_t				*di_data;
	off_t				di_roffset;
	off_t				di_woffset;
	atomic_t			di_free;	/* buffer free entry count */
	struct semaphore	di_sem;
	wait_queue_head_t	di_rqueue;	/* unsatisfied read from device sleeps on it */
	struct cdev			di_cdev;	/* char device structure */
	int ref;
	int					di_irq;		/*
									 * Set on first open. Reset on _release
									 * Thus serving as an indication of the
									 * device being open
									 */
} di_t;

#define	IMASKED		0x1

typedef struct dihndlr {
	void (*dih_func)(unsigned);
	char dih_masked;		/* the interrupt was masked */
	di_t *dih_dev;			/* pointer to the device structure assigned at init() */
} dih_t;


void
docint_mask_interrupt_handler(unsigned int index);


#endif
#define	MAX_CAUSE	(32 * (REMOTE_MAC_INT_GROUP + 1))
#endif
