#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "docint.h"
#include "hal_interrupt_cause.h"
#include "docsis_logger.h"
#include "logger.h"
#include "status.h"


/**************************************************************************/
/*! \fn STATUS HAL_IsrMaskCause(Uint16 icause, Int8 mask)
 **************************************************************************
 *  \brief Masks or unmasks a given interrupt, according to cause-number, 
 *  \param[in] icause - cause number for the given interrupt.
 *  \param[in] mask - defines if to mask (True) or unmask (False) the interrupt.
 *  \return OK/NOK                                     
 **************************************************************************/
STATUS HAL_IsrMaskCause(Uint16 icause, Int8 mask)
{
	static Int32 fd = -1;
	ii_t info;

	if (fd < 0)
		if ((fd = open("/dev/" DI_NAME "0", O_RDWR)) < 0) {
            TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, " Can't open device.\n");
			return STATUS_NOK;
		}

	info.ii_inum = icause;
	info.ii_masked = mask;
    info.ii_clear = 0;

	if (write(fd, &info, sizeof(ii_t)) != sizeof(ii_t)) {
        TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, " Can't write to DOCINT device, errno %d.\n", errno);
			return STATUS_NOK;
	}

	return STATUS_OK;
}

/**************************************************************************/
/*! \fn STATUS HAL_IsrClearIrq(Uint16 irq)
 **************************************************************************
 *  \brief Clears a given IRQ
 *  \param[in] irq
 *  \return OK/NOK                                     
 **************************************************************************/
STATUS HAL_IsrClearIrq(Uint16 irq)
{
    static Int32 fd = -1;
    ii_t info;
    
    if (fd < 0)
        if ((fd = open("/dev/" DI_NAME "0", O_RDWR)) < 0) {
            TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, " Can't open device.\n");
            return STATUS_NOK;
        }
        
        info.ii_inum = irq;
        info.ii_masked = 0;
        info.ii_clear = 1;
        
        if (write(fd, &info, sizeof(ii_t)) != sizeof(ii_t)) {
            TI_LOG_ERROR(TI_COMPONENT_DOCSIS, DOCSIS_MOD_HAL, 0, " Can't write to DOCINT device, errno %d.\n", errno);
            return STATUS_NOK;
        }
        
        return STATUS_OK;
}


/*
int mask_cause(short icause, char mask)
{
	static int fd = -1;
	ii_t info;

	if (fd < 0)
		if ((fd = open("/dev/" DI_NAME "0", O_RDWR)) < 0) {
			fprintf(stderr, "Can't open device\n");
			return(-1);
		}

	info.ii_inum = icause;
	info.ii_masked = mask;

	if (write(fd, &info, sizeof(ii_t)) != sizeof(ii_t)) {
			fprintf(stderr, "Can't write to DOCINT device, errno %d \n", errno);
			return(-2);
	}

	return(0);
}
*/
/**************************************************************************/
/*! \fn STATUS HAL_IsrMaskBitField(Uint32 bitField, Bool mask, Uint32 intGroup)
 **************************************************************************
 *  \brief Masks or unmasks a series of interrupts in a given interrupt-group, 
 *  \according to a given Bit-Field.
 *  \param[in] bitField - every bit that is set to 1 indicates an interrupt within 
 *  \                     the group that needs to be masked or unmasked.
 *  \param[in] mask - defines if to mask (True) or unmask (False) the interrupts.
 *  \param[in] intGroup - defines the interrupt-group that the interrupts belong to.
 *  \return OK/NOK                                     
 **************************************************************************/
STATUS HAL_IsrMaskBitField(Uint32 bitField, Bool mask, Uint32 intGroup)
{
    Uint32 index;

    if(intGroup > REMOTE_MAC_INT_GROUP)
        return STATUS_NOK;

    for(index = 0; index < 32; index++)
    {                             
        if(bitField & (1<<index))
        {
            HAL_IsrMaskCause(HAL_GET_INTERRUPT_CAUSE(intGroup,index),mask);
        }
    }

    return STATUS_OK;
}
