/*
 *
 * soc_interface_driver.h
 * Description:
 * declaration of functions and types used in the SoC Interface Driver.
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _SOC_INTERFACE_DRIVER_H_
#define _SOC_INTERFACE_DRIVER_H_

#include "_tistdtypes.h"
#include "sys_ptypes.h"
#include "baby_puma_driver.h"
#include "secondary_puma_driver.h"

/**************************************************************************/
/*      INTERFACE  Defines and Structs                                    */
/**************************************************************************/
#define SOC_INTERFACE_DRIVER_NAME     "soc_if_driver"


#define SECONDARY_PUMA_TEST
/*****************************************/
/* Definitions                           */
/*****************************************/

typedef struct RemoteRegInfo
{
   Uint32   reg_addr;
   Uint16   reg_value;
}RemoteRegInfo_t;

typedef struct RemoteMacRegInfo
{
    Uint32   reg_addr;
    Uint32   reg_value;
}RemoteMacRegInfo_t;

#define MAX_MAC_COPY_BYTES 512
typedef struct RemoteMacCopyInfo
{
    Uint32   regOffset;
    Uint32   copyLen;
    Uint8    data[MAX_MAC_COPY_BYTES];
}RemoteMacCopyInfo_t;


/*****************************************/
/* Definitions for DOCSIS SoC-Level Init */
/*****************************************/

#define PUMA5_SOC_BASE_ADDRESS                  IO_PHY2VIRT(0x08600000)
#define REMOTE_DEVICE_SOC_ADDR                  ((Uint32) ( (secondaryPumaIsInitialized()) ? secondaryPumaGetSoCAddr() : babyPumaGetSoCAddr() )) 
#define GET_SOC_REG_ADDR( device, offset )      ((Uint32) ( device ? (REMOTE_DEVICE_SOC_ADDR) + offset : PUMA5_SOC_BASE_ADDRESS + offset ) )\

/* SoC MPEG Out Registers */
#define PUMA5_MPEG_OUT_BASE_ADDR          	   IO_PHY2VIRT(0x06000000)
#define REMOTE_DEVICE_MPEG_OUT_ADDR            ((Uint32) ( (secondaryPumaIsInitialized()) ? secondaryPumaGetDynamic1Addr() : babyPumaGetMpegOutAddr() )) 
#define GET_MPEG_OUT_REG_ADDR( device, offset ) ((Uint32) (device ? (REMOTE_DEVICE_MPEG_OUT_ADDR) + offset : PUMA5_MPEG_OUT_BASE_ADDR + offset ) )

#define PUMA5_BOOTCFG_KICK_0                    IO_PHY2VIRT(0x08611A38)
#define PUMA5_BOOTCFG_KICK_1                    IO_PHY2VIRT(0x08611A3C)

#define PUMA5_BOOTCFG_KICK_0_VAL                (0x83E70B13)
#define PUMA5_BOOTCFG_KICK_1_VAL                (0x95A4F1E0)

#define PUMA5_PSC_GO                            IO_PHY2VIRT(0x08621120)
#define PUMA5_PSC_GO_VAL                        (0x00000001)

#define PUMA5_BOOTCFG_KICK_0_OFFSET             (0x11A38)
#define PUMA5_BOOTCFG_KICK_1_OFFSET             (0x11A3C)

#define PUMA5_PUDCR0_REG_OFFSER                 (0x11B60)
#define PUMA5_PUDCR0_REG_VALUE                  (0x01)

#define PUMA5_BOOTCFG_PUDCR0_REG( device )      GET_SOC_REG_ADDR( device, PUMA5_PUDCR0_REG_OFFSER )

#define PUMA5_BOOTCFG_KICK_0_REG( device )      GET_SOC_REG_ADDR( device, PUMA5_BOOTCFG_KICK_0_OFFSET )
#define PUMA5_BOOTCFG_KICK_1_REG( device )      GET_SOC_REG_ADDR( device, PUMA5_BOOTCFG_KICK_1_OFFSET )

#define PUMA5_DOCSIS_PSC_OFFSET                 (0x21A2C) /* 0x08621a2c */
#define PUMA5_DOCSIS_PSC_REG( device )          GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_PSC_OFFSET )

#define PUMA5_PSC_GO_OFFSET                     (0x21120) /* 0x08621120 */
#define PUMA5_PSC_GO_REG( device )              GET_SOC_REG_ADDR( device, PUMA5_PSC_GO_OFFSET )

#define PUMA5_PINMUX0_OFFSET                    (0x11B10)
#define PUMA5_PINMUX0_REG( device )             GET_SOC_REG_ADDR( device, PUMA5_PINMUX0_OFFSET )

#define PUMA5_BOOTCFG_CPPI_SEL_OFFSET           (0x11B98)
#define PUMA5_BOOTCFG_CPPI_SEL_REG( device )    GET_SOC_REG_ADDR( device, PUMA5_BOOTCFG_CPPI_SEL_OFFSET )


/**********************/
/* MPEG-IN definitions*/
/**********************/
#define PUMA5_PINMUX0_MPEG_IN_EN          (0x2)



#define PUMA5_DOCSIS_AUX_GPIO_OUT_VAL_OFFSET    (0x1092C)
#define PUMA5_DOCSIS_AUX_GPIO_DIR_OFFSET        (0x10930)
#define PUMA5_DOCSIS_AUX_GPIO_OFFSET            (0x10934)

#define PUMA5_DOCSIS_GPIO_OUT_VAL_OFFSET        (0x10904)
#define PUMA5_DOCSIS_GPIO_DIR_OFFSET            (0x10908)
#define PUMA5_DOCSIS_GPIO_OFFSET                (0x1090c)


#define PUMA5_DOCSIS_AUX_GPIO_OUT_VAL_REG( device ) GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_AUX_GPIO_OUT_VAL_OFFSET )
#define PUMA5_DOCSIS_AUX_GPIO_DIR_REG( device )     GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_AUX_GPIO_DIR_OFFSET )
#define PUMA5_DOCSIS_AUX_GPIO_REG( device )         GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_AUX_GPIO_OFFSET )

#define PUMA5_DOCSIS_GPIO_OUT_VAL_REG( device )     GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_GPIO_OUT_VAL_OFFSET )
#define PUMA5_DOCSIS_GPIO_DIR_REG( device )         GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_GPIO_DIR_OFFSET )
#define PUMA5_DOCSIS_GPIO_REG( device )             GET_SOC_REG_ADDR( device, PUMA5_DOCSIS_GPIO_OFFSET )


typedef enum
{
    SOC_DEVICE_LOCAL = 0,
    SOC_DEVICE_REMOTE = 1,

} socDevice_e;


/**************************************************************************/
/*! \fn void socWriteRegister (unsigned long  addr, unsigned long  value, bool printEn)
 **************************************************************************
 *  \brief Write SoC Register.
 *  \param[in] addr    - Register addr.
 *  \param[in] value   - Value to be written.
 *  \param[in] printEn - If true a debug print is enable.
 *  \return none.
 **************************************************************************/
void socWriteRegister (Uint32  addr, Uint32  value, Bool printEn);

/**************************************************************************/
/*! \fn void socReadRegister(unsigned long  addr, volatile unsigned long *value, bool printEn)
 **************************************************************************
 *  \brief Read SoC Register.
 *  \param[in] addr    - Register addr.
 *  \param[in] *value  - Pointer for returned register value.
 *  \param[in] printEn - If true a debug print is enable.
 *  \return none.
 **************************************************************************/
void socReadRegister (Uint32  addr, Uint32 *value, Bool printEn);

/**************************************************************************/
/*! \fn void socBitField32Set(Uint32  *addr, Uint32  data, Uint32  offset, Uint32  width)
 **************************************************************************
 *  \brief Set specific bits in a 32 bit variable.
 *  \param[in] *addr   - pointer to 32 bit variable to change.
 *  \param[in] data    - Data to write.
 *  \param[in] offset  - Write from this offset.
 *  \param[in] width   - Write width bits.
 *  \return none.
 **************************************************************************/
void socBitField32Set(Uint32  *addr, Uint32  data, Uint32  offset,Uint32  width);
#endif
